﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;
using System.Windows.Forms;
using System.Xml;
using System.Web;

namespace WordpressPlugin
{
    public class CommentBackgroundWorker : BackgroundWorker
    {
        private CommentTabPage commentTabPage;
        private BlogContext blogContext;

        public CommentBackgroundWorker(CommentTabPage commentTabPage, BlogContext blogContext)
        {
            this.commentTabPage = commentTabPage;
            this.commentTabPage.Loading = true;

            this.blogContext = blogContext;

            this.DoWork += new DoWorkEventHandler(DoWorkHandler);
            this.RunWorkerCompleted += new RunWorkerCompletedEventHandler(RunWorkerCompletedHandler);
        }

        private void DoWorkHandler(object sender, DoWorkEventArgs e)
        {
            // Make sure not to touch the UI in this method because its run in a background thread.
            // Any exceptions thrown will be automatically caught and saved for the RunWorkerCompleted event.
            e.Result = XmlRpcHelper.WpGetComments(blogContext, commentTabPage.Status.ToString());
        }

        private void RunWorkerCompletedHandler(object sender, RunWorkerCompletedEventArgs e)
        {
            // Make sure there was no exception and that the background thread wasn't cancelled.
            if (e.Error == null && !e.Cancelled)
            {
                // Find the the list of comments
                XmlDocument doc = (XmlDocument)e.Result;
                XmlNode comments = doc.SelectSingleNode("//methodResponse/params/param/value/array/data");

                // Look through all the comments in the array and add them to the form
                foreach (XmlNode node in comments.ChildNodes)
                {
                    Comment c = new Comment();

                    XmlNodeList properties = node.SelectNodes("./struct/member");

                    // Add each of the properties to the comment
                    foreach (XmlNode prop in properties)
                    {
                        string key = XmlRpcHelper.GetInnerTextSafe(prop.SelectSingleNode("./name"));
                        string value = XmlRpcHelper.GetInnerTextSafe(prop.SelectSingleNode("./value"));

                        switch (key)
                        {
                            case "date_created_gmt":
                                c.Date = value;
                                break;
                            case "comment_id":
                                c.CommandId = value;
                                break;
                            case "content":
                                c.Content = value;
                                break;
                            case "link":
                                c.Link = value;
                                break;
                            case "post_title":
                                c.PostTitle = HttpUtility.HtmlDecode(value);
                                break;
                            case "author":
                                c.Author = value;
                                break;
                            case "author_url":
                                c.AuthorUrl = value;
                                break;
                            case "author_email":
                                c.AuthorEmail = value;
                                break;
                        }
                    }

                    // Set the status on our own, since we know what we queeried for
                    c.Status = commentTabPage.Status;

                    // Create a CommentControl to display the comment to the user
                    CommentControl control = new CommentControl(c);

                    // Resize the UI control a little thinner to make room for a scrollbar
                    control.DeleteClicked += (s, ev) =>
                    {
                        if (Comment.ChangeCommentStatus(blogContext, c, CommentStatus.delete))
                            commentTabPage.RemoveComment(control);
                    };
                    control.ApproveClicked += (s, ev) =>
                    {
                        if (Comment.ChangeCommentStatus(blogContext, c, CommentStatus.approve))
                            commentTabPage.RemoveComment(control);
                    };
                    control.SpamClicked += (s, ev) =>
                    {
                        if (Comment.ChangeCommentStatus(blogContext, c, CommentStatus.spam))
                            commentTabPage.RemoveComment(control);
                    };

                    // Add to the form
                    commentTabPage.AddComment(control);
                }
            }

            commentTabPage.Loading = false;
        }
    }
}
