﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace WordpressPlugin
{
    /// <summary>
    /// UI Control that shows all the information about a comment
    /// </summary>
    public partial class CommentControl : UserControl
    {
        /// <summary>
        /// The comment that this UI control will display
        /// </summary>
        public readonly Comment Comment;

        public CommentControl()
        {
            InitializeComponent();
        }

        public CommentControl(Comment comment)
        {
            InitializeComponent();

            Comment = comment;

            // Set up all the information for each comment
            labelComment.Text = comment.Content;
            labelHeading.Text = comment.PostTitle;            
            labelSubheading.Text = comment.Author + "  (" + comment.AuthorEmail + ")";

            // Open the link to the blog post when the user clicks the link
            labelHeading.Click += (s, e) => { Process.Start(comment.Link); };

            // Hide the action buttons that dont apply
            if (comment.Status == CommentStatus.spam)
                linkLabelSpam.Visible = false;

            if (comment.Status == CommentStatus.approve)
                linkLabelApprove.Visible = false;
        }

        public event EventHandler DeleteClicked;
        private void FireDeleteClicked()
        {
            if (DeleteClicked != null)
                DeleteClicked(this, EventArgs.Empty);
        }

        public event EventHandler SpamClicked;
        private void FireSpamClicked()
        {
            if (SpamClicked != null)
                SpamClicked(this, EventArgs.Empty);
        }

        public event EventHandler ApproveClicked;
        private void FireApproveClicked()
        {
            if (ApproveClicked != null)
                ApproveClicked(this, EventArgs.Empty);
        }

        private void linkLabelApprove_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            FireApproveClicked();
        }

        private void linkLabelSpam_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            FireSpamClicked();
        }

        private void linkLabelDelete_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            FireDeleteClicked();
        }

        private void CommentControl_Resize(object sender, EventArgs e)
        {
            ResizeToFitComment();
        }

        private void labelComment_TextChanged(object sender, EventArgs e)
        {
            ResizeToFitComment();
        }

        private void ResizeToFitComment()
        {
            // Resize the UI control to fit the whole comment
            Size size = TextRenderer.MeasureText(labelComment.Text, labelComment.Font, new Size(labelComment.Width, Int32.MaxValue), TextFormatFlags.WordBreak);
            labelComment.Height = size.Height;
            Height = labelComment.Bottom + 10;
        }
    }
}
