﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WordpressPlugin
{
    public partial class CustomFieldsForm : Form
    {
        public CustomFieldsForm()
        {
            InitializeComponent();
        }

        public CustomFieldsForm(BindingList<CustomField> customFields)
        {
            InitializeComponent();



            SetupDataGridView(customFields);

            // Sort the recently used custom fields so the most recent are first
            int count = 0;
            List<KeyValuePair<string, int>> recentCustomFields = new List<KeyValuePair<string, int>>(CustomFieldsPlugin.RecentCustomFields);

            recentCustomFields.Sort((firstPair, nextPair) =>
            {
                return nextPair.Value.CompareTo(firstPair.Value);
            });
            
            // Add the last 30 recent custom fields to the quick insert panel
            foreach (KeyValuePair<string, int> customFieldKey in recentCustomFields)
            {
                LinkLabel ll = new LinkLabel();
                ll.AutoSize = true;
                ll.Text = customFieldKey.Key;
                ll.ForeColor = Color.RoyalBlue;
                ll.Click += new EventHandler(ll_Click);
                flowLayoutPanelRecent.Controls.Add(ll);
                count++;

                if(count == 30)
                    break;
            }
            
        }

        private void SetupDataGridView(BindingList<CustomField> customFields)
        {
            dataGridViewFields.AutoGenerateColumns = true;
            dataGridViewFields.AllowUserToAddRows = true;
            dataGridViewFields.AllowUserToDeleteRows = true;
            dataGridViewFields.EditMode = DataGridViewEditMode.EditOnEnter;
            dataGridViewFields.DataSource = customFields;
            dataGridViewFields.Columns[0].ReadOnly = true;
            dataGridViewFields.Columns[0].Visible = false;
            dataGridViewFields.Columns[2].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
        }

        void ll_Click(object sender, EventArgs e)
        {
            // The user clicked a recent custom field link, add it to the list
            // so it will show up in the data grid view
            CustomField cf = new CustomField();
            cf.Key = ((LinkLabel)sender).Text;
            cf.Value = "";
            (dataGridViewFields.DataSource as BindingList<CustomField>).Add(cf);
        }

        private void CustomFieldsForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Save any pending edits to a custom field
            dataGridViewFields.EndEdit();
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            // Expand the form to show the textbox that has PHP code for the custom field
            using (new AnchorSuspension(dataGridViewFields, checkBoxPHP, textBoxPHPCode))
            {
                if (checkBoxPHP.Checked)
                {
                    ClientSize = new Size(ClientSize.Width, textBoxPHPCode.Bottom + 10);
                }
                else
                {
                    ClientSize = new Size(ClientSize.Width, textBoxPHPCode.Top - 1);
                }

                UpdatePHPCode();
            }
        }

        /// <summary>
        /// A class that lets you expand the form with anchored controls with no impact
        /// </summary>
        private class AnchorSuspension : IDisposable
        {
            private Control[] _controls;
            private readonly AnchorStyles[] _anchorStyles;

            public AnchorSuspension(params Control[] controls)
            {
                _anchorStyles = new AnchorStyles[controls.Length];
                _controls = controls;
                for (int i = 0; i < _controls.Length; i++)
                {
                    Control c = _controls[i];
                    if (c != null)
                    {
                        _anchorStyles[i] = c.Anchor;
                        c.Anchor = AnchorStyles.Left | AnchorStyles.Top;
                    }
                }
            }

            public void Dispose()
            {
                Control[] controls = _controls;
                _controls = null;
                if (controls != null)
                {
                    for (int i = 0; i < controls.Length; i++)
                    {
                        Control c = controls[i];
                        if (c != null)
                        {
                            c.Anchor = _anchorStyles[i];
                        }
                    }
                }
            }
        }

        private void dataGridViewFields_CellContentClick(object sender, DataGridViewCellEventArgs e)
        {
            UpdatePHPCode();
        }

        private void UpdatePHPCode()
        {
            
            string key = "";
            // Look to see if we have a currently selected row
            if (dataGridViewFields.CurrentRow != null)
            {
                if (dataGridViewFields.CurrentRow.Cells.Count == 3 && dataGridViewFields.CurrentRow.Cells[1].Value != null)
                    key = dataGridViewFields.CurrentRow.Cells[1].Value.ToString();
            }
            // Create some PHP code to show the custom field
            textBoxPHPCode.Text = String.Format("<?php ${1} = get_post_meta($post->ID, '{0}', true); ?>\r\n<?php if(${1}) echo ${1}; ?>", key, key.Replace(" ", "").ToLower());
        }

        private void dataGridViewFields_CurrentCellChanged(object sender, EventArgs e)
        {
            UpdatePHPCode();
        }

        private void dataGridViewFields_KeyPress(object sender, KeyPressEventArgs e)
        {
            UpdatePHPCode();
        }

        private void buttonOK_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            Close();
        }
    }
}
