﻿using System;
using System.Collections.Generic;
using System.Text;
using WindowsLive.Writer.Api;
using System.Xml;
using System.ComponentModel;
using System.Windows.Forms;

namespace WordpressPlugin
{
    [WriterPluginAttribute("385FDF11-76DE-4a94-957B-E42E2BD82F11", "Custom Fields")]
    public class CustomFieldsPlugin : PublishNotificationHook
    {
        public override void OnPostPublish(System.Windows.Forms.IWin32Window dialogOwner, IProperties properties, IPublishingContext publishingContext, bool publish)
        {
            BlogContext context = new BlogContext(publishingContext);            

            XmlNode custom_fields_member_node = null;
            BindingList<CustomField> custom_fields = new BindingList<CustomField>();
            XmlDocument doc = null;
            try
            {
                doc = XmlRpcHelper.MetaWeblogGetPost(context);
            }
            catch (OperationCanceledException)
            {
                return;
            }

            XmlNodeList members = doc.SelectNodes("//methodResponse/params/param/value/struct/member");
            foreach (XmlNode member in members)
            {
                if (XmlRpcHelper.GetInnerTextSafe(member.SelectSingleNode("./name")) == "custom_fields")
                {
                    custom_fields_member_node = member;
                    XmlNodeList fields = member.SelectNodes("./value/array/data/value/struct");
                    foreach (XmlNode field in fields)
                    {
                        CustomField cf = new CustomField();
                        foreach (XmlNode prop in field.ChildNodes)
                        {
                            string propName = XmlRpcHelper.GetInnerTextSafe(prop.SelectSingleNode("./name"));
                            string propValue = XmlRpcHelper.GetInnerTextSafe(prop.SelectSingleNode("./value"));

                            switch (propName)
                            {
                                case "id":
                                    cf.Id = propValue;
                                    break;
                                case "key":
                                    cf.Key = propValue;
                                    break;
                                case "value":
                                    cf.Value = propValue;
                                    break;
                            }
                        }
                        if(!cf.Key.StartsWith("_"))
                            custom_fields.Add(cf);
                    }
                }
            }

            using (CustomFieldsForm cff = new CustomFieldsForm(custom_fields))
            {
                if (cff.ShowDialog() == DialogResult.Cancel)
                    return;
            }

            List<CustomField> finalCustomFields = new List<CustomField>();
            for (int i = 0; i < custom_fields.Count; i++)
            {
                if (custom_fields[i] == null || string.IsNullOrEmpty(custom_fields[i].Key))
                    continue;
                finalCustomFields.Add(custom_fields[i]);
            }

            foreach (CustomField cf in finalCustomFields)
            {
                RecentCustomFields[cf.Key] = (int)(DateTime.Now.Ticks / 100000000000000);
            }

            if (finalCustomFields.Count == 0)
                return;

            SaveRecentCustomFieldsFromProperties();

            StringBuilder sb = new StringBuilder();
            sb.Append("<name>custom_fields</name><value><array><data>");

            foreach (CustomField cf in finalCustomFields)
            {
                sb.Append("<value><struct>");
                if (!string.IsNullOrEmpty(cf.Id))
                    sb.AppendFormat("<member><name>id</name><value><string>{0}</string></value></member>", cf.Id);

                sb.AppendFormat("<member><name>key</name><value><string>{0}</string></value></member>", cf.Key);
                sb.AppendFormat("<member><name>value</name><value><string>{0}</string></value></member>", cf.Value);
                sb.Append("</struct></value>");
            }

            sb.Append("</data></array></value>");
            custom_fields_member_node.InnerXml = sb.ToString();

            string postXml = doc.SelectSingleNode("//methodResponse/params/param").OuterXml;

            try
            {
                XmlRpcHelper.MetaWeblogEditPost(context, postXml);
            }
            catch (OperationCanceledException)
            {
                return;
            }

        }

        public override void Initialize(IProperties pluginOptions)
        {
            base.Initialize(pluginOptions);

            LoadRecentCustomFieldsFromProperties();
        }

        public void LoadRecentCustomFieldsFromProperties()
        {
            if (RecentCustomFields.Count == 0)
            {
                IProperties recentCustomFields = Options.GetSubProperties("recent_custom_fields");
                   
                foreach(string subprop in recentCustomFields.Names)
                {
                    RecentCustomFields.Add(subprop, recentCustomFields.GetInt(subprop, 0));
                }
                
            }
        }

        public void SaveRecentCustomFieldsFromProperties()
        {
            IProperties recentCustomFields = Options.GetSubProperties("recent_custom_fields");
            foreach(KeyValuePair<string, int> field in RecentCustomFields)
            {
                recentCustomFields.SetInt(field.Key, field.Value);
            }
        }

        internal static readonly SortedDictionary<string, int> RecentCustomFields = new SortedDictionary<string, int>();
    }

    public class CustomField
    {
        public string Id
        {
            get;
            set;
        }
        public string Key
        {
            get;
            set;
        }
        public string Value
        {
            get;
            set;
        }
    }
}
